<?php
namespace I18n;

class StringExtractor extends \StringExtractor
{
    protected $domain;

    public function __construct(string $domain, array $rules = [])
    {
        parent::__construct($rules);
        $this->domain = $domain;
    }

    function extract_from_directory( $dir, $excludes = array(), $includes = array(), $prefix = '' ) {
        $old_cwd = getcwd();
        chdir( $dir );
        $translations = new \Translations;
        $file_names = (array) scandir( '.' );
        foreach ( $file_names as $file_name ) {
            if ( '.' == $file_name || '..' == $file_name ) continue;
            if ( preg_match( '/\.(php|twig)$/', $file_name ) && $this->does_file_name_match( $prefix . $file_name, $excludes, $includes ) ) {
                $extracted = $this->extract_from_file( $file_name, $prefix );
                $translations->merge_originals_with( $extracted );
            }
            if ( is_dir( $file_name ) ) {
                $extracted = $this->extract_from_directory( $file_name, $excludes, $includes, $prefix . $file_name . '/' );
                $translations->merge_originals_with( $extracted );
            }
        }
        chdir( $old_cwd );
        return $translations;
    }

    function extract_from_file( $file_name, $prefix ) {
        $code = file_get_contents( $file_name );

        if (preg_match('/\.twig$/', $file_name)) {
            $translations = $this->extract_from_twig($code, $prefix . $file_name);
        } else {
            $translations = $this->extract_from_code( $code, $prefix . $file_name );
        }

        return $translations;
    }

    function entry_from_call($call, $file_name)
    {
        switch ($call['name']) {
            case '_':
            case '__':
            case '_e':
            case 'esc_attr__':
            case 'esc_attr_e':
            case 'esc_html__':
            case 'esc_html_e':
            case '_c':
            default:
                $domainIndex = 1;
                break;
            case '_x':
            case '_ex':
            case 'esc_attr_x':
            case 'esc_html_x':
            case '_n_noop':
                $domainIndex = 2;
                break;
            case '_n':
            case '_nx_noop':
            case '_nc':
                $domainIndex = 3;
                break;
            case '_nx':
                $domainIndex = 4;
                break;
        }

        if (!isset($call['args'][$domainIndex]) || $call['args'][$domainIndex] !== $this->domain) {
            return false;
        }

        return parent::entry_from_call($call, $file_name);
    }

    function extract_from_twig($code, $file_name)
    {
        $translations = new \Translations;
        $function_calls = $this->find_function_calls_in_twig( array_keys( $this->rules ), $code );

        foreach( $function_calls as $call ) {
            $entry = $this->entry_from_call( $call, $file_name );
            if ( is_array( $entry ) )
                foreach( $entry as $single_entry )
                    $translations->add_entry_or_merge( $single_entry );
            elseif ( $entry)
                $translations->add_entry_or_merge( $entry );
        }
        return $translations;
    }

    /**
     * Finds all function calls in $code and returns an array with an associative array for each function:
     *	- name - name of the function
     *	- args - array for the function arguments. Each string literal is represented by itself, other arguments are represented by null.
     *  - line - line number
     */
    function find_function_calls_in_twig( $function_names, $code ) {
        $lines = explode(PHP_EOL, $code);
        $calls = [];

        foreach ($function_names as $function_name) {
            foreach ($lines as $line_no => $line) {
                $regex = sprintf('/{{ %s\((.+)\) }}/', $function_name);

                if (preg_match_all($regex, $line, $matches)) {
                    foreach ($matches[1] as $match) {
                        $calls[] = [
                            'name' => $function_name,
                            'args' => array_map(function($item) {
                                return trim($item, ' \'"');
                            }, explode(',', $match)),
                            'line' => $line_no + 1,
                        ];
                    }
                }
            }
        }

        return $calls;
    }
}